{**********************************************}
{                                              }
{             FastReport v3.0                  }
{      SQL Direct enduser components           }
{                                              }
{         Copyright (c) 1998-2006              }
{           by Fast Reports Inc.               }
{                                              }
{ Copyright (c) 2001-2006 by Stalker SoftWare  }
{                                              }
{**********************************************}
 
unit frxSDComponents;

interface

{$I frx.inc}

uses
  Windows, Classes, SysUtils, frxClass, frxCustomDB, DB, Graphics, SDEngine,
  frxDsgnIntf, frxRes, frxFormUtils
{$IFDEF Delphi6}
, Variants
{$ENDIF}
{$IFDEF QBUILDER}
, fqbClass
{$ENDIF};

type
  TfrxSDComponents = class(TfrxDBComponents)
  public
    function GetDescription: String; override;
  end; { TfrxSDComponents }


  { TfrxSDDataBase }

  TfrxSDDataBase = class(TfrxCustomDatabase)
  private
    FDatabase: TSDDataBase;
    function GetRemoteDatabase: String;
    procedure SetRemoteDatabase(const Value: String);
    function GetServerType: TSDServerType;
    procedure SetServerType(const Value: TSDServerType);
    function GetKeepConnection: Boolean;
    procedure SetKeepConnection(const Value: Boolean);
    function GetTransIsolation: TSDTransIsolation;
    procedure SetTransIsolation(const Value: TSDTransIsolation);
    function GetIdleTimeOut: Integer;
    procedure SetIdleTimeOut(const Value: Integer);
    function GetSessionName: string;
    procedure SetSessionName(const Value: string);

  protected
    procedure SetConnected(Value: Boolean); override;
    procedure SetDatabaseName(const Value: String); override;
    procedure SetLoginPrompt(Value: Boolean); override;
    procedure SetParams(Value: TStrings); override;
    function GetConnected: Boolean; override;
    function GetDatabaseName: String; override;
    function GetLoginPrompt: Boolean; override;
    function GetParams: TStrings; override;

  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    property Database: TSDDatabase read FDatabase;

    procedure StartTransaction;
    procedure Commit;
    procedure Rollback;

  published
    property DatabaseName: String read GetDatabaseName write SetDatabaseName;
    property KeepConnection :Boolean read GetKeepConnection write SetKeepConnection default True;
    property LoginPrompt: Boolean read GetLoginPrompt write SetLoginPrompt default True;
    property Params: TStrings read GetParams write SetParams;
    property RemoteDatabase :String read GetRemoteDatabase write SetRemoteDatabase;
    property ServerType :TSDServerType read GetServerType write SetServerType default stODBC;
    property TransIsolation :TSDTransIsolation read GetTransIsolation write SetTransIsolation default tiReadCommitted;
    property IdleTimeOut :Integer read GetIdleTimeOut write SetIdleTimeOut default 0;
    property SessionName: string read GetSessionName write SetSessionName;
    property Connected: Boolean read GetConnected write SetConnected default False;
  end; { TfrxSDDataBase }


  TfrxDataSetNotifyEvent = String[63];
  TfrxUpdateRecordEvent  = String[63];
  TfrxFilterRecordEvent  = String[63];

  { TfrxSDTable }

  TfrxSDTable = class(TfrxCustomTable)
  private
    FTable :TSDTable;
    procedure SetDatabaseName(const Value: String);
    function GetDatabaseName: String;
    procedure SetSessionName(const Value: String);
    function GetSessionName: String;
    function GetReadOnly: Boolean;
    procedure SetReadOnly(const Value: Boolean);
    function GetDBSession: TSDSession;
    function GetAutoCalcFields: Boolean;
    function GetDefaultIndex: Boolean;
    function GetDetachOnFetchAll: Boolean;
    procedure SetAutoCalcFields(const Value: Boolean);
    procedure SetDefaultIndex(const Value: Boolean);
    procedure SetDetachOnFetchAll(const Value: Boolean);
    function GetCachedUpdates: Boolean;
    procedure SetCachedUpdates(const Value: Boolean);

  protected
    procedure SetName(const AName: TComponentName); override;
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetMasterFields(const Value: String); override;
    procedure SetIndexName(const Value: String); override;
    function GetIndexName: String; override;
    function GetTableName: String; override;
    procedure SetTableName(const Value: String); override;
    function GetIndexFieldNames: String; override;
    procedure SetIndexFieldNames(const Value: String); override;

  public
    constructor Create(AOwner: TComponent); override;
    class function GetDescription: String; override;
    property Table: TSDTable read FTable;
    property DBSession :TSDSession read GetDBSession;

    function LocateNext(const KeyFields: string; const KeyValues: Variant;Options: TLocateOptions): Boolean;
    procedure OpenEmpty;
    procedure EnableControls;
    procedure DisableControls;
    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;

  published
    property DatabaseName     :String  read GetDatabaseName     write SetDatabaseName;
    property SessionName      :String  read GetSessionName      write SetSessionName;
    property IndexName        :String  read GetIndexName        write SetIndexName;
    property IndexFieldNames  :String  read GetIndexFieldNames  write SetIndexFieldNames;
    property TableName        :String  read GetTableName        write SetTableName;
    property ReadOnly         :Boolean read GetReadOnly         write SetReadOnly default False;
    property DefaultIndex     :Boolean read GetDefaultIndex     write SetDefaultIndex default True;
    property AutoCalcFields   :Boolean read GetAutoCalcFields   write SetAutoCalcFields default True;
    property DetachOnFetchAll :Boolean read GetDetachOnFetchAll write SetDetachOnFetchAll default False;
    property CachedUpdates    :Boolean read GetCachedUpdates    write SetCachedUpdates default False;
    property MasterFields;
    property Active default False;

  end; { TfrxSDTable }

  { TfrxSDQuery }

  TfrxSDQuery = class(TfrxCustomQuery)
  private
    FQuery          :TSDQuery;
    FAfterOpen      :TfrxDataSetNotifyEvent;
    FBeforeOpen     :TfrxDataSetNotifyEvent;
    FAfterScroll    :TfrxDataSetNotifyEvent;
    FBeforeScroll   :TfrxDataSetNotifyEvent;
    FOnUpdateRecord :TfrxUpdateRecordEvent;
    FOnFilterRecord :TfrxFilterRecordEvent;

    function GetParamCheck: Boolean;
    procedure SetParamCheck(const Value: Boolean);
    function GetUniDirectional: Boolean;
    procedure SetUniDirectional(const Value: Boolean);
    procedure SetDatabaseName(Value: String);
    function GetDatabaseName: String;
    procedure SetSessionName(const Value: String);
    function GetSessionName: String;
    function GetAutoCalcFields: Boolean;
    function GetDetachOnFetchAll: Boolean;
    procedure SetAutoCalcFields(const Value: Boolean);
    procedure SetDetachOnFetchAll(const Value: Boolean);
    function GetDBSession: TSDSession;
    function GetRequestLive: Boolean;
    procedure SetRequestLive(const Value: Boolean);
    function GetCachedUpdates: Boolean;
    procedure SetCachedUpdates(const Value: Boolean);

    procedure DoAfterOpen(DataSet: TDataSet);
    procedure DoBeforeOpen(DataSet: TDataSet);
    procedure DoAfterScroll(DataSet: TDataSet);
    procedure DoBeforeScroll(DataSet: TDataSet);
    procedure DoUpdateRecord(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
    procedure DoFilterRecord(DataSet: TDataSet; var Accept: Boolean);

  protected
    procedure SetName(const AName: TComponentName); override;
    procedure SetSQL(Value: TStrings); override;
    function GetSQL: TStrings; override;
    procedure SetMaster(const Value: TDataSource); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure UpdateParams; override;

    function LocateNext(const KeyFields: string; const KeyValues: Variant;Options: TLocateOptions): Boolean;
    procedure OpenEmpty;
    procedure ExecSQL;
    procedure FetchAll;
    procedure RefreshRecordEx(const cRefreshSQL: string);
    procedure EnableControls;
    procedure DisableControls;
    procedure SortRecords(const AFields, AscOrder, ACaseSensitive: string);
    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;

    procedure SetUpdateRecordEvent();
    procedure RemoveUpdateRecordEvent();

    property Query: TSDQuery read FQuery;
    property DBSession :TSDSession read GetDBSession;

{$IFDEF QBUILDER}
    function QBEngine: TfqbEngine; override;
{$ENDIF}

  published
    property ParamCheck       :Boolean read GetParamCheck       write SetParamCheck default True;
    property UniDirectional   :Boolean read GetUniDirectional   write SetUniDirectional default False;
    property DatabaseName     :String  read GetDatabaseName     write SetDatabaseName;
    property SessionName      :String  read GetSessionName      write SetSessionName;
    property AutoCalcFields   :Boolean read GetAutoCalcFields   write SetAutoCalcFields default True;
    property DetachOnFetchAll :Boolean read GetDetachOnFetchAll write SetDetachOnFetchAll default False;
    property RequestLive      :Boolean read GetRequestLive      write SetRequestLive default False;
    property CachedUpdates    :Boolean read GetCachedUpdates    write SetCachedUpdates default True;
    property Active default False;

    property AfterOpen      :TfrxDataSetNotifyEvent read FAfterOpen      write FAfterOpen;
    property BeforeOpen     :TfrxDataSetNotifyEvent read FBeforeOpen     write FBeforeOpen;
    property AfterScroll    :TfrxDataSetNotifyEvent read FAfterScroll    write FAfterScroll;
    property BeforeScroll   :TfrxDataSetNotifyEvent read FBeforeScroll   write FBeforeScroll;
    property OnUpdateRecord :TfrxUpdateRecordEvent  read FOnUpdateRecord write FOnUpdateRecord;
    property OnFilterRecord :TfrxFilterRecordEvent  read FOnFilterRecord write FOnFilterRecord;

  end; { TfrxSDQuery }


  { TfrxSDMacroQuery }

  TfrxSDMacroQuery = class(TfrxCustomQuery)
  private
    FQuery          :TSDMacroQuery;
    FMacroParams    :TfrxParams;
    FAfterOpen      :TfrxDataSetNotifyEvent;
    FBeforeOpen     :TfrxDataSetNotifyEvent;
    FAfterScroll    :TfrxDataSetNotifyEvent;
    FBeforeScroll   :TfrxDataSetNotifyEvent;
    FOnUpdateRecord :TfrxUpdateRecordEvent;
    FOnFilterRecord :TfrxFilterRecordEvent;

    function GetParamCheck: Boolean;
    procedure SetParamCheck(const Value: Boolean);
    function GetUniDirectional: Boolean;
    procedure SetUniDirectional(const Value: Boolean);
    procedure SetDatabaseName(Value: String);
    function GetDatabaseName: String;
    procedure SetSessionName(const Value: String);
    function GetSessionName: String;
    function GetAutoCalcFields: Boolean;
    function GetDetachOnFetchAll: Boolean;
    procedure SetAutoCalcFields(const Value: Boolean);
    procedure SetDetachOnFetchAll(const Value: Boolean);
    function GetDBSession: TSDSession;
    function GetRequestLive: Boolean;
    procedure SetRequestLive(const Value: Boolean);
    function GetMacroChar: Char;
    procedure SetMacroChar(const Value: Char);
    procedure SetMacroParams(const Value: TfrxParams);
    procedure ReadData(Reader: TReader);
    procedure WriteData(Writer: TWriter);
    function GetCachedUpdates: Boolean;
    procedure SetCachedUpdates(const Value: Boolean);

    procedure DoAfterOpen(DataSet: TDataSet);
    procedure DoBeforeOpen(DataSet: TDataSet);
    procedure DoAfterScroll(DataSet: TDataSet);
    procedure DoBeforeScroll(DataSet: TDataSet);
    procedure DoUpdateRecord(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
    procedure DoFilterRecord(DataSet: TDataSet; var Accept: Boolean);

    procedure UpdateMacros();
    function GetMacroCount(): Word;

  protected
    procedure SetName(const AName: TComponentName); override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure SetSQL(Value: TStrings); override;
    function GetSQL: TStrings; override;
    procedure OnChangeSQL(Sender: TObject); override;
    procedure SetMaster(const Value: TDataSource); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure UpdateParams; override;

    function LocateNext(const KeyFields: string; const KeyValues: Variant;Options: TLocateOptions): Boolean;
    procedure OpenEmpty;
    procedure ExpandMacros;
    procedure ExecSQL;
    function MacroByName(const MacroName: String): TfrxParamItem;
    procedure FetchAll;
    procedure RefreshRecordEx(const cRefreshSQL: string);
    procedure EnableControls;
    procedure DisableControls;
    procedure SortRecords(const AFields, AscOrder, ACaseSensitive: string);
    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;

    procedure SetUpdateRecordEvent();
    procedure RemoveUpdateRecordEvent();

    property MacroCount :Word read GetMacroCount;

    property Query: TSDMacroQuery read FQuery;
    property DBSession :TSDSession read GetDBSession;

{$IFDEF QBUILDER}
    function QBEngine: TfqbEngine; override;
{$ENDIF}

  published
    property ParamCheck       :Boolean    read GetParamCheck       write SetParamCheck default True;
    property UniDirectional   :Boolean    read GetUniDirectional   write SetUniDirectional default False;
    property DatabaseName     :String     read GetDatabaseName     write SetDatabaseName;
    property SessionName      :String     read GetSessionName      write SetSessionName;
    property AutoCalcFields   :Boolean    read GetAutoCalcFields   write SetAutoCalcFields default True;
    property DetachOnFetchAll :Boolean    read GetDetachOnFetchAll write SetDetachOnFetchAll default False;
    property RequestLive      :Boolean    read GetRequestLive      write SetRequestLive default False;
    property MacroChar        :Char       read GetMacroChar        write SetMacroChar default '%';
    property Macros           :TfrxParams read FMacroParams        write SetMacroParams;
    property CachedUpdates    :Boolean    read GetCachedUpdates    write SetCachedUpdates default True;
    property Active default False;

    property AfterOpen      :TfrxDataSetNotifyEvent read FAfterOpen      write FAfterOpen;
    property BeforeOpen     :TfrxDataSetNotifyEvent read FBeforeOpen     write FBeforeOpen;
    property AfterScroll    :TfrxDataSetNotifyEvent read FAfterScroll    write FAfterScroll;
    property BeforeScroll   :TfrxDataSetNotifyEvent read FBeforeScroll   write FBeforeScroll;
    property OnUpdateRecord :TfrxUpdateRecordEvent  read FOnUpdateRecord write FOnUpdateRecord;
    property OnFilterRecord :TfrxFilterRecordEvent  read FOnFilterRecord write FOnFilterRecord;

  end; { TfrxSDMacroQuery }

{$IFDEF QBUILDER}
  TfrxEngineSD = class(TfqbEngine)
  private
    FQuery :TSDQuery;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadTableList(ATableList :TStrings); override;
    procedure ReadFieldList(const ATableName :String; var AFieldList :TfqbFieldList); override;
    function ResultDataSet: TDataSet; override;
    procedure SetSQL(const Value :String); override;

    property Query :TSDQuery read FQuery;
  end; { TfqbEngine }
{$ENDIF}

 procedure frxMacroParamsToTMacroParams(Query: TfrxSDMacroQuery; Macros: TParams);

implementation

{$R frxSD.RES}

uses
  frxSDRTTI
{$IFNDEF NO_EDITORS}
 ,frxSDEditor
{$ENDIF}
;

{ TfrxSDComponents }

function TfrxSDComponents.GetDescription: String;
begin
 Result := 'SQL Direct';
end; { GetDescription }


{ TfrxSDDataBase }

constructor TfrxSDDataBase.Create(AOwner: TComponent);
begin
 inherited;
 FDatabase := TSDDatabase.Create(nil);
 Component := FDatabase;
 FDatabase.ServerType := stODBC;
end; { Create }

class function TfrxSDDataBase.GetDescription: String;
begin
 Result := 'TfrxSDDataBase';
end; { GetDescription }

procedure TfrxSDDataBase.SetConnected(Value: Boolean);
begin
 BeforeConnect(Value); // for FR 3.20.1
 FDatabase.Connected := Value;
end;

function TfrxSDDataBase.GetConnected: Boolean;
begin
 Result := FDatabase.Connected;
end; { GetConnected }

function TfrxSDDataBase.GetDatabaseName: String;
begin
 Result := FDatabase.DatabaseName;
end; { GetDatabaseName }

procedure TfrxSDDataBase.SetDatabaseName(const Value: String);
begin
 FDatabase.DatabaseName := Value;
end; { SetDatabaseName }

procedure TfrxSDDataBase.SetLoginPrompt(Value: Boolean);
begin
 FDatabase.LoginPrompt := Value;
end; { SetLoginPrompt }

function TfrxSDDataBase.GetLoginPrompt: Boolean;
begin
 Result := FDatabase.LoginPrompt;
end; { GetLoginPromp }

procedure TfrxSDDataBase.SetParams(Value: TStrings);
begin
 FDatabase.Params := Value;
end; { SetParams }

function TfrxSDDataBase.GetParams: TStrings;
begin
 Result := FDatabase.Params;
end; { GetParams }

function TfrxSDDataBase.GetRemoteDatabase: String;
begin
 Result := FDatabase.RemoteDatabase;
end; { GetRemoteDatabase }

procedure TfrxSDDataBase.SetRemoteDatabase(const Value: String);
begin
 FDatabase.RemoteDatabase := Value;
end; { SetRemoteDatabase }

function TfrxSDDataBase.GetServerType: TSDServerType;
begin
 Result := FDatabase.ServerType;
end; { GetServerType }

procedure TfrxSDDataBase.SetServerType(const Value: TSDServerType);
begin
 FDatabase.ServerType := Value;
end; { SetServerType }

function TfrxSDDataBase.GetKeepConnection: Boolean;
begin
 Result := FDatabase.KeepConnection;
end; { GetKeepConnection }

procedure TfrxSDDataBase.SetKeepConnection(const Value: Boolean);
begin
 FDatabase.KeepConnection := Value;
end; { SetKeepConnection }

function TfrxSDDataBase.GetTransIsolation: TSDTransIsolation;
begin
 Result := FDatabase.TransIsolation;
end; { GetTransIsolation }

procedure TfrxSDDataBase.SetTransIsolation(const Value: TSDTransIsolation);
begin
 FDatabase.TransIsolation := Value;
end; { SetTransIsolation }

function TfrxSDDataBase.GetIdleTimeOut: Integer;
begin
 Result := FDatabase.IdleTimeOut;
end; { GetIdleTimeOut }

procedure TfrxSDDataBase.SetIdleTimeOut(const Value: Integer);
begin
 FDatabase.IdleTimeOut := Value;
end; { SetIdleTimeOut }

function TfrxSDDataBase.GetSessionName: string;
begin
 Result := FDatabase.SessionName;
end; { GetSessionName }

procedure TfrxSDDataBase.SetSessionName(const Value: string);
begin
 FDatabase.SessionName := Value;
end; { SetSessionName }

procedure TfrxSDDataBase.StartTransaction;
begin
 FDatabase.StartTransaction;
end; { StartTransaction }

procedure TfrxSDDataBase.Commit;
begin
 FDatabase.Commit;
end; { Commit }

procedure TfrxSDDataBase.Rollback;
begin
 FDatabase.Rollback;
end; { Rollback }


{ TfrxSDTable }

constructor TfrxSDTable.Create(AOwner: TComponent);
begin
 FTable := TSDTable.Create(nil);
 DataSet := FTable;
 inherited;
end; { Create }

class function TfrxSDTable.GetDescription: String;
begin
 Result := 'TfrxSDTable';
end; { GetDescription }

function TfrxSDTable.GetDatabaseName: String;
begin
 Result := FTable.DatabaseName;
end; { GetDatabaseName }

procedure TfrxSDTable.SetDatabaseName(const Value: String);
begin
 FTable.DatabaseName := Value;
end; { SetDatabaseName }

function TfrxSDTable.GetSessionName: String;
begin
 Result := FTable.SessionName;
end; { GetSessionName }

procedure TfrxSDTable.SetSessionName(const Value: String);
begin
 FTable.SessionName := Value;
end; { SetSessionName }

function TfrxSDTable.GetIndexName: String;
begin
 Result := FTable.IndexName;
end; { GetIndexName }

function TfrxSDTable.GetTableName: String;
begin
 Result := FTable.TableName;
end; { GetTableName }

procedure TfrxSDTable.SetIndexName(const Value: String);
begin
 FTable.IndexName := Value;
end; { SetIndexName }

procedure TfrxSDTable.SetTableName(const Value: String);
begin
 FTable.TableName := Value;
end; { SetTableName }

function TfrxSDTable.GetIndexFieldNames: String;
begin
 Result := FTable.IndexFieldNames;
end; { GetIndexFieldNames }

procedure TfrxSDTable.SetIndexFieldNames(const Value: String);
begin
 FTable.IndexFieldNames := Value;
end; { SetIndexFieldNames }

function TfrxSDTable.GetReadOnly: Boolean;
begin
 Result := FTable.ReadOnly;
end; { GetReadOnly }

procedure TfrxSDTable.SetReadOnly(const Value: Boolean);
begin
 FTable.ReadOnly := Value;
end; { SetReadOnly }

function TfrxSDTable.GetDBSession: TSDSession;
begin
 Result := FTable.DBSession;
end; { GetDBSession }

function TfrxSDTable.LocateNext(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
 Result := FTable.LocateNext(KeyFields, KeyValues, Options);
end; { LocateNext }

function TfrxSDTable.GetAutoCalcFields: Boolean;
begin
 Result := FTable.AutoCalcFields;
end; { GetAutoCalcFields }

procedure TfrxSDTable.SetAutoCalcFields(const Value: Boolean);
begin
 FTable.AutoCalcFields := Value;
end; { SetAutoCalcFields }

function TfrxSDTable.GetDefaultIndex: Boolean;
begin
 Result := FTable.DefaultIndex;
end; { GetDefaultIndex }

procedure TfrxSDTable.SetDefaultIndex(const Value: Boolean);
begin
 FTable.DefaultIndex := Value;
end; { SetDefaultIndex }

function TfrxSDTable.GetDetachOnFetchAll: Boolean;
begin
 Result := FTable.DetachOnFetchAll;
end; { GetDetachOnFetchAll }

procedure TfrxSDTable.SetDetachOnFetchAll(const Value: Boolean);
begin
 FTable.DetachOnFetchAll := Value;
end; { SetDetachOnFetchAll }

procedure TfrxSDTable.OpenEmpty;
begin
 FTable.OpenEmpty;
end; { OpenEmpty }

procedure TfrxSDTable.SetName(const AName: TComponentName);
begin
 inherited;
 FTable.Name := AName;
end; { SetName }

procedure TfrxSDTable.SetMaster(const Value: TDataSource);
begin
 FTable.MasterSource := Value;
end; { SetMaster }

procedure TfrxSDTable.SetMasterFields(const Value: String);
begin
 FTable.MasterFields := Value;
end; { SetMasterFields }

procedure TfrxSDTable.DisableControls;
begin
 FTable.DisableControls;
end; { DisableControls }

procedure TfrxSDTable.EnableControls;
begin
 FTable.EnableControls;
end; { EnableControls }

function TfrxSDTable.CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
begin
 Result := FTable.CreateBlobStream(Field, Mode);
end; { CreateBlobStream }

function TfrxSDTable.GetCachedUpdates: Boolean;
begin
 Result := FTable.CachedUpdates;
end; { GetCachedUpdates }

procedure TfrxSDTable.SetCachedUpdates(const Value: Boolean);
begin
 FTable.CachedUpdates := Value;
end; { SetCachedUpdates }

{ TfrxSDQuery }

constructor TfrxSDQuery.Create(AOwner: TComponent);
begin

 FQuery := TSDQuery.Create(nil);
 Dataset := FQuery;

 FQuery.AfterOpen      := DoAfterOpen;
 FQuery.BeforeOpen     := DoBeforeOpen;
 FQuery.AfterScroll    := DoAfterScroll;
 FQuery.BeforeScroll   := DoBeforeScroll;
 FQuery.OnFilterRecord := DoFilterRecord;

 inherited;

end; { Create }

destructor TfrxSDQuery.Destroy;
begin
 inherited;
end; { Destroy }

class function TfrxSDQuery.GetDescription: String;
begin
 Result := 'TfrxSDQuery';
end; { GetDescription }

function TfrxSDQuery.GetSQL: TStrings;
begin
 Result := FQuery.SQL;
end; { GetSQL }

procedure TfrxSDQuery.SetSQL(Value: TStrings);
begin
 FQuery.SQL.Assign(Value);
end; { SetSQL }

procedure TfrxSDQuery.UpdateParams;
begin
 frxParamsToTParams(Self, FQuery.Params);
end; { UpdateParams }

function TfrxSDQuery.GetParamCheck: Boolean;
begin
 Result := FQuery.ParamCheck;
end; { GetParamCheck }

procedure TfrxSDQuery.SetParamCheck(const Value: Boolean);
begin
 FQuery.ParamCheck := Value;
end; { SetParamCheck }

function TfrxSDQuery.GetUniDirectional: Boolean;
begin
 Result := FQuery.UniDirectional;
end; { GetUniDirectional }

procedure TfrxSDQuery.SetUniDirectional(const Value: Boolean);
begin
 FQuery.UniDirectional := Value;
end; { SetUniDirectional }

function TfrxSDQuery.GetDatabaseName: String;
begin
 Result := FQuery.DatabaseName;
end; { GetDatabaseName }

procedure TfrxSDQuery.SetDatabaseName(Value: String);
begin
 FQuery.DatabaseName := Value;
end; { SetDatabaseName }

function TfrxSDQuery.GetSessionName: String;
begin
 Result := FQuery.SessionName;
end; { GetSessionName }

procedure TfrxSDQuery.SetSessionName(const Value: String);
begin
 FQuery.SessionName := Value;
end; { SetSessionName }

function TfrxSDQuery.GetAutoCalcFields: Boolean;
begin
 Result := FQuery.AutoCalcFields;
end; { GetAutoCalcFields }

procedure TfrxSDQuery.SetAutoCalcFields(const Value: Boolean);
begin
 FQuery.AutoCalcFields := Value;
end; { SetAutoCalcFields }

function TfrxSDQuery.GetDetachOnFetchAll: Boolean;
begin
 Result := FQuery.DetachOnFetchAll
end; { GetDetachOnFetchAll }

procedure TfrxSDQuery.SetDetachOnFetchAll(const Value: Boolean);
begin
 FQuery.DetachOnFetchAll := Value;
end; { SetDetachOnFetchAll }

function TfrxSDQuery.GetDBSession: TSDSession;
begin
 Result := FQuery.DBSession;
end; { GetDBSession }

function TfrxSDQuery.GetRequestLive: Boolean;
begin
 Result := FQuery.RequestLive;
end; { GetRequestLive }

procedure TfrxSDQuery.SetRequestLive(const Value: Boolean);
begin
 FQuery.RequestLive := Value;
end; { SetRequestLive }

function TfrxSDQuery.LocateNext(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
 Result := FQuery.LocateNext(KeyFields, KeyValues, Options);
end; { LocateNext }

procedure TfrxSDQuery.OpenEmpty;
begin
 FQuery.OpenEmpty;
end; { OpenEmpty }

procedure TfrxSDQuery.ExecSQL;
begin
 FQuery.ExecSQL;
end; { ExecSQL }

procedure TfrxSDQuery.FetchAll;
begin
 FQuery.FetchAll;
end; { FetchAll }

procedure TfrxSDQuery.SetName(const AName: TComponentName);
begin
 inherited;
 FQuery.Name := AName;
end; { SetName }

procedure TfrxSDQuery.SetMaster(const Value: TDataSource);
begin
 FQuery.DataSource := Value;
end;{ SetMaster }

{$IFDEF QBUILDER}
function TfrxSDQuery.QBEngine: TfqbEngine;
begin
 Result := TfrxEngineSD.Create(nil);
 TfrxEngineSD(Result).Query.DatabaseName := FQuery.DatabaseName;
end; { QBEngine }
{$ENDIF}

procedure TfrxSDQuery.RefreshRecordEx(const cRefreshSQL: string);
begin
 FQuery.RefreshRecordEx(cRefreshSQL);
end; { RefreshRecordEx }

procedure TfrxSDQuery.DisableControls;
begin
 FQuery.DisableControls;
end; { DisableControls }

procedure TfrxSDQuery.EnableControls;
begin
 FQuery.EnableControls;
end; { EnableControls }

procedure TfrxSDQuery.SortRecords(const AFields, AscOrder, ACaseSensitive :String);
begin
 FQuery.SortRecords(AFields, AscOrder, ACaseSensitive)
end; { SortRecords }

function TfrxSDQuery.CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
begin
 Result := FQuery.CreateBlobStream(Field, Mode);
end; { CreateBlobStream }

function TfrxSDQuery.GetCachedUpdates: Boolean;
begin
 Result := FQuery.CachedUpdates;
end; { GetCachedUpdates }

procedure TfrxSDQuery.SetCachedUpdates(const Value: Boolean);
begin
 FQuery.CachedUpdates := Value;
end; { SetCachedUpdates }

procedure TfrxSDQuery.SetUpdateRecordEvent;
begin
 FQuery.OnUpdateRecord := DoUpdateRecord;
end; { SetUpdateRecordEvent }

procedure TfrxSDQuery.RemoveUpdateRecordEvent;
begin
 FQuery.OnUpdateRecord := nil;
end; { RemoveUpdateRecordEvent }

procedure TfrxSDQuery.DoAfterOpen(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FAfterOpen <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FAfterOpen, vArr);

 end; { if }

end; { DoAfterOpen }

procedure TfrxSDQuery.DoBeforeOpen(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FBeforeOpen <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FBeforeOpen, vArr);

 end; { if }

end; { DoBeforeOpen }

procedure TfrxSDQuery.DoAfterScroll(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FAfterScroll <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FAfterScroll, vArr);

 end; { if }

end; { DoAfterScroll }

procedure TfrxSDQuery.DoBeforeScroll(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FBeforeScroll <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FBeforeScroll, vArr);

 end; { if }

end; { DoBeforeScroll }

procedure TfrxSDQuery.DoUpdateRecord(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
var
  vArr: Variant;

begin

 if FOnUpdateRecord <> '' then begin

   vArr := VarArrayOf([Integer(DataSet), UpdateKind, UpdateAction]);

   if Report <> nil then
     Report.DoParamEvent(FOnUpdateRecord, vArr);

   UpdateAction := vArr[2];

 end; { if }

end; { DoUpdateRecord }

procedure TfrxSDQuery.DoFilterRecord(DataSet: TDataSet; var Accept: Boolean);
var
  vArr: Variant;

begin

 if FOnFilterRecord <> '' then begin

   vArr := VarArrayOf([Integer(DataSet), Accept]);

   if Report <> nil then
     Report.DoParamEvent(FOnFilterRecord, vArr);

   Accept := vArr[1];

 end; { if }

end; { DoFilterRecord }


{ TfrxSDMacroQuery }

constructor TfrxSDMacroQuery.Create(AOwner: TComponent);
begin

 FMacroParams := TfrxParams.Create;
 FQuery := TSDMacroQuery.Create(nil);
 Dataset := FQuery;

 FQuery.AfterOpen      := DoAfterOpen;
 FQuery.BeforeOpen     := DoBeforeOpen;
 FQuery.AfterScroll    := DoAfterScroll;
 FQuery.BeforeScroll   := DoBeforeScroll;
 FQuery.OnFilterRecord := DoFilterRecord;

 inherited;

end; { Create }

destructor TfrxSDMacroQuery.Destroy;
begin
 FMacroParams.Free;
 inherited;
end; { Destroy }

class function TfrxSDMacroQuery.GetDescription: String;
begin
 Result := 'TfrxSDMacroQuery';
end; { GetDescription }

function TfrxSDMacroQuery.GetSQL: TStrings;
begin
 Result := FQuery.SQL;
end; { GetSQL }

procedure TfrxSDMacroQuery.SetSQL(Value: TStrings);
begin
 FQuery.SQL.Assign(Value);
end; { SetSQL }

procedure TfrxSDMacroQuery.UpdateParams;
begin
 frxParamsToTParams(Self, FQuery.Params);
 frxMacroParamsToTMacroParams(Self, FQuery.Macros);
end; { UpdateParams }

procedure TfrxSDMacroQuery.DefineProperties(Filer: TFiler);
begin
 inherited;
 Filer.DefineProperty('Macross', ReadData, WriteData, True);
end; { DefineProperties }

procedure TfrxSDMacroQuery.ReadData(Reader: TReader);
begin
 frxReadCollection(FMacroParams, Reader, Self);
 UpdateMacros();
end; { ReadData }

procedure TfrxSDMacroQuery.WriteData(Writer: TWriter);
begin
 frxWriteCollection(FMacroParams, Writer, Self);
end; { WriteData }

procedure TfrxSDMacroQuery.OnChangeSQL(Sender: TObject);
begin

 inherited;
 UpdateMacros();

end; { OnChangeSQL }

procedure TfrxSDMacroQuery.UpdateMacros();
var
  i, j: Integer;
  NewParams: TfrxParams;

begin

 NewParams := TfrxParams.Create;
 try

   for i := 0 to FQuery.MacroCount - 1 do
     with NewParams.Add do begin
       Name := FQuery.Macros[i].Name;
       j := Macros.IndexOf(Name);
       if j <> -1 then begin
         DataType   := Macros.Items[j].DataType;
         Value      := Macros.Items[j].Value;
         Expression := Macros.Items[j].Expression;
       end; { if }
     end; { for }

 finally

   Macros.Assign(NewParams);
   NewParams.Free;

 end; { try }

end; { UpdateMacros }

function TfrxSDMacroQuery.GetParamCheck: Boolean;
begin
 Result := FQuery.ParamCheck;
end; { GetParamCheck }

procedure TfrxSDMacroQuery.SetParamCheck(const Value: Boolean);
begin
 FQuery.ParamCheck := Value;
end; { SetParamCheck }

function TfrxSDMacroQuery.GetUniDirectional: Boolean;
begin
 Result := FQuery.UniDirectional;
end; { GetUniDirectional }

procedure TfrxSDMacroQuery.SetUniDirectional(const Value: Boolean);
begin
 FQuery.UniDirectional := Value;
end; { SetUniDirectional }

function TfrxSDMacroQuery.GetDatabaseName: String;
begin
 Result := FQuery.DatabaseName;
end; { GetDatabaseName }

procedure TfrxSDMacroQuery.SetDatabaseName(Value: String);
begin
 FQuery.DatabaseName := Value;
end; { SetDatabaseName }

function TfrxSDMacroQuery.GetSessionName: String;
begin
 Result := FQuery.SessionName;
end; { GetSessionName }

procedure TfrxSDMacroQuery.SetSessionName(const Value: String);
begin
 FQuery.SessionName := Value;
end; { SetSessionName }

function TfrxSDMacroQuery.GetAutoCalcFields: Boolean;
begin
 Result := FQuery.AutoCalcFields;
end; { GetAutoCalcFields }

procedure TfrxSDMacroQuery.SetAutoCalcFields(const Value: Boolean);
begin
 FQuery.AutoCalcFields := Value;
end; { SetAutoCalcFields }

function TfrxSDMacroQuery.GetDetachOnFetchAll: Boolean;
begin
 Result := FQuery.DetachOnFetchAll
end; { GetDetachOnFetchAll }

procedure TfrxSDMacroQuery.SetDetachOnFetchAll(const Value: Boolean);
begin
 FQuery.DetachOnFetchAll := Value;
end; { SetDetachOnFetchAll }

function TfrxSDMacroQuery.GetDBSession: TSDSession;
begin
 Result := FQuery.DBSession;
end; { GetDBSession }

function TfrxSDMacroQuery.GetRequestLive: Boolean;
begin
 Result := FQuery.RequestLive;
end; { GetRequestLive }

procedure TfrxSDMacroQuery.SetRequestLive(const Value: Boolean);
begin
 FQuery.RequestLive := Value;
end; { SetRequestLive }

function TfrxSDMacroQuery.LocateNext(const KeyFields: string; const KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
 Result := FQuery.LocateNext(KeyFields, KeyValues, Options);
end; { LocateNext }

procedure TfrxSDMacroQuery.OpenEmpty;
begin
 FQuery.OpenEmpty;
end; { OpenEmpty }

procedure TfrxSDMacroQuery.SetMacroChar(const Value: Char);
begin
 FQuery.MacroChar := Value;
end; { SetMacroChar }

function TfrxSDMacroQuery.GetMacroChar: Char;
begin
 Result := FQuery.MacroChar;
end; { GetMacroChar }

procedure TfrxSDMacroQuery.SetMacroParams(const Value: TfrxParams);
begin
 FMacroParams.Assign(Value);
end; { SetMacroParams }

procedure TfrxSDMacroQuery.ExpandMacros;
begin
 FQuery.ExpandMacros;
end; { ExpandMacros }

function TfrxSDMacroQuery.GetMacroCount() :Word;
begin
 Result := FQuery.MacroCount;
end; { GetMacroCount }

procedure TfrxSDMacroQuery.ExecSQL;
begin
 FQuery.ExecSQL;
end; { ExecSQL }

procedure TfrxSDMacroQuery.FetchAll;
begin
 FQuery.FetchAll;
end; { FetchAll }

function TfrxSDMacroQuery.MacroByName(const MacroName: String): TfrxParamItem;
begin
 Result := FMacroParams.Find(MacroName);
end; { MacroByName }

procedure TfrxSDMacroQuery.SetName(const AName: TComponentName);
begin
 inherited;
 FQuery.Name := AName;
end; { SetName }

procedure TfrxSDMacroQuery.SetMaster(const Value: TDataSource);
begin
 FQuery.DataSource := Value;
end;{ SetMaster }

{$IFDEF QBUILDER}
function TfrxSDMacroQuery.QBEngine: TfqbEngine;
begin
 Result := TfrxEngineSD.Create(nil);
 TfrxEngineSD(Result).Query.DatabaseName := FQuery.DatabaseName;
end; { QBEngine }
{$ENDIF}

procedure TfrxSDMacroQuery.RefreshRecordEx(const cRefreshSQL: string);
begin
 FQuery.RefreshRecordEx(cRefreshSQL);
end; { RefreshRecordEx }

procedure TfrxSDMacroQuery.DisableControls();
begin
 FQuery.DisableControls;
end; { DisableControls }

procedure TfrxSDMacroQuery.EnableControls();
begin
 FQuery.EnableControls;
end; { EnableControls }

function TfrxSDMacroQuery.GetCachedUpdates: Boolean;
begin
 Result := FQuery.CachedUpdates;
end; { GetCachedUpdates }

procedure TfrxSDMacroQuery.SetCachedUpdates(const Value: Boolean);
begin
 FQuery.CachedUpdates := Value;
end; { SetCachedUpdates }

procedure TfrxSDMacroQuery.SortRecords(const AFields, AscOrder, ACaseSensitive :String);
begin
 FQuery.SortRecords(AFields, AscOrder, ACaseSensitive)
end; { SortRecords }

function TfrxSDMacroQuery.CreateBlobStream(Field :TField; Mode :TBlobStreamMode) :TStream;
begin
 Result := FQuery.CreateBlobStream(Field, Mode);
end; { CreateBlobStream }

procedure TfrxSDMacroQuery.SetUpdateRecordEvent();
begin
 FQuery.OnUpdateRecord := DoUpdateRecord;
end; { SetUpdateRecordEvent }

procedure TfrxSDMacroQuery.RemoveUpdateRecordEvent();
begin
 FQuery.OnUpdateRecord := nil;
end; { RemoveUpdateRecordEvent }

procedure TfrxSDMacroQuery.DoAfterOpen(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FAfterOpen <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FAfterOpen, vArr);

 end; { if }

end; { DoAfterOpen }

procedure TfrxSDMacroQuery.DoBeforeOpen(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FBeforeOpen <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FBeforeOpen, vArr);

 end; { if }

end; { DoBeforeOpen }

procedure TfrxSDMacroQuery.DoAfterScroll(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FAfterScroll <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FAfterScroll, vArr);

 end; { if }

end; { DoAfterScroll }

procedure TfrxSDMacroQuery.DoBeforeScroll(DataSet: TDataSet);
var
  vArr: Variant;

begin

 if FBeforeScroll <> '' then begin

   vArr := VarArrayOf([Integer(DataSet)]);

   if Report <> nil then
     Report.DoParamEvent(FBeforeScroll, vArr);

 end; { if }

end; { DoBeforeScroll }

procedure TfrxSDMacroQuery.DoUpdateRecord(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
var
  vArr: Variant;

begin

 if FOnUpdateRecord <> '' then begin

   vArr := VarArrayOf([Integer(DataSet), UpdateKind, UpdateAction]);

   if Report <> nil then
     Report.DoParamEvent(FOnUpdateRecord, vArr);

   UpdateAction := vArr[2];

 end; { if }

end; { DoUpdateRecord }

procedure TfrxSDMacroQuery.DoFilterRecord(DataSet: TDataSet; var Accept: Boolean);
var
  vArr: Variant;

begin

 if FOnFilterRecord <> '' then begin

   vArr := VarArrayOf([Integer(DataSet), Accept]);

   if Report <> nil then
     Report.DoParamEvent(FOnFilterRecord, vArr);

   Accept := vArr[1];

 end; { if }

end; { DoFilterRecord }


{ TfrxEngineSD }

{$IFDEF QBUILDER}
constructor TfrxEngineSD.Create(AOwner: TComponent);
begin
 inherited;
 FQuery := TSDQuery.Create(Self);
end; { Create }

destructor TfrxEngineSD.Destroy;
begin
 FQuery.Free;
 inherited
end; { Destroy }

procedure TfrxEngineSD.ReadFieldList(const ATableName :String; var AFieldList :TfqbFieldList);
var
  TempTable :TSDQuery;
  Fields    :TFieldDefs;
  i         :Integer;
  tmpField  :TfqbField;

begin

 AFieldList.Clear;
 TempTable := TSDQuery.Create(Self);
 try

   TempTable.DatabaseName := FQuery.DatabaseName;
   TempTable.SQL.Text := 'select * from '+ATableName;
   Fields := TempTable.FieldDefs;

   try

     TempTable.Active := True;
     tmpField:= TfqbField(AFieldList.Add);
     tmpField.FieldName := '*';

     for i := 0 to Fields.Count - 1 do begin
       tmpField := TfqbField(AFieldList.Add);
       tmpField.FieldName := Fields.Items[i].Name;
       tmpField.FieldType := Ord(Fields.Items[i].DataType)
     end; { for }

   except
   end; { try ... except }

 finally
   TempTable.Free;
 end; { try .. finally }

end; { ReadFieldList }

procedure TfrxEngineSD.ReadTableList(ATableList: TStrings);
begin

 ATableList.BeginUpdate;
 ATableList.Clear;
 try
   Session.GetTableNames(FQuery.DatabaseName, '', ShowSystemTables, ATableList);
 finally
   ATableList.EndUpdate;
 end; { try }

end; { ReadTableList }

function TfrxEngineSD.ResultDataSet: TDataSet;
begin
 Result := FQuery;
end; { ResultDataSet }

procedure TfrxEngineSD.SetSQL(const Value: string);
begin
 FQuery.SQL.Text := Value;
end; { SetSQL }
{$ENDIF}

{ frxMacroParamsToTMacroParams }

procedure frxMacroParamsToTMacroParams(Query: TfrxSDMacroQuery; Macros: TParams);
var
  i: Integer;
  Item: TfrxParamItem;

begin

 for i := 0 to Macros.Count - 1 do
   if Query.Macros.IndexOf(Macros[i].Name) <> -1 then begin
     Item := Query.Macros[Query.Macros.IndexOf(Macros[i].Name)];
     Macros[i].Clear;
     // Bound should be True in design mode
     if not (Query.IsLoading or Query.IsDesigning) then
       Macros[i].Bound := False
     else
       Macros[i].Bound := True;
     Macros[i].DataType := Item.DataType;
     if Trim(Item.Expression) <> '' then
       if not (Query.IsLoading or Query.IsDesigning) then
         if Query.Report <> nil then begin
           Query.Report.CurObject := Query.Name;
           Item.Value := Query.Report.Calc(Item.Expression);
         end; { if }
     if not VarIsEmpty(Item.Value) then begin
       Macros[i].Bound := True;
       Macros[i].Text := VarToStr(Item.Value);
     end; { if }
   end; { if }

end; { frxMacroParamsToTMacroParams }

var
  Bmp1 :TBitmap;
  Bmp2 :TBitmap;
  Bmp3 :TBitmap;
  Bmp4 :TBitmap;
  Bmp5 :TBitmap;

initialization
  frxResources.Add('TfrxDataSetNotifyEvent',
    'PascalScript=(DataSet: TDataSet);' + #13#10 +
    'C++Script=(TDataSet DataSet)' + #13#10 +
    'BasicScript=(DataSet)' + #13#10 +
    'JScript=(DataSet)');
  frxResources.Add('TfrxUpdateRecordEvent',
    'PascalScript=(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);' + #13#10 +
    'C++Script=(TDataSet DataSet; TUpdateKind UpdateKind; TUpdateAction &UpdateAction)' + #13#10 +
    'BasicScript=(DataSet, UpdateKind, byref UpdateAction)' + #13#10 +
    'JScript=(DataSet, UpdateKind, byref UpdateAction)');
  frxResources.Add('TfrxFilterRecordEvent',
    'PascalScript=(DataSet: TDataSet; var Accept: Boolean);' + #13#10 +
    'C++Script=(TDataSet DataSet; Boolean &Accept)' + #13#10 +
    'BasicScript=(DataSet, byref Accept)' + #13#10 +
    'JScript=(DataSet, byref Accept)');


  Bmp1 := TBitmap.Create;
  Bmp1.LoadFromResourceName(hInstance, 'FRXSDCOMPONENTS');
  frxObjects.RegisterCategory('SD', Bmp1, 'SQL Direct');

  Bmp2 := TBitmap.Create;
  Bmp2.LoadFromResourceName(hInstance, 'FRX_SDDB');
  frxObjects.RegisterObject1(TfrxSDDataBase, Bmp2, 'SQL Direct: frxSDDataBase', 'SD');

  Bmp3 := TBitmap.Create;
  Bmp3.LoadFromResourceName(hInstance, 'FRX_SDQUERY');
  frxObjects.RegisterObject1(TfrxSDQuery, Bmp3, 'SQL Direct: frxSDQuery', 'SD');

  Bmp4 := TBitmap.Create;
  Bmp4.LoadFromResourceName(hInstance, 'FRX_SDTABLE');
  frxObjects.RegisterObject1(TfrxSDTable, Bmp4, 'SQL Direct: frxSDTable', 'SD');

  Bmp5 := TBitmap.Create;
  Bmp5.LoadFromResourceName(hInstance, 'FRX_SDMACROQUERY');
  frxObjects.RegisterObject1(TfrxSDMacroQuery, Bmp5, 'SQL Direct: frxSDMacroQuery', 'SD');

finalization
  frxObjects.UnRegister(TfrxSDDataBase);
  frxObjects.UnRegister(TfrxSDQuery);
  frxObjects.UnRegister(TfrxSDTable);
  frxObjects.UnRegister(TfrxSDMacroQuery);
  Bmp1.Free;
  Bmp2.Free;
  Bmp3.Free;
  Bmp4.Free;
  Bmp5.Free;

end.