{**********************************************}
{                                              }
{             FastReport v3.0                  }
{      SD components design editors            }
{                                              }
{         Copyright (c) 1998-2006              }
{           by Fast Reports Inc.               }
{                                              }
{ Copyright (c) 2001-2006 by Stalker SoftWare  }
{                                              }
{**********************************************}

unit frxSDEditor;

interface

uses
  Windows, Classes, SysUtils, Forms, Controls,  
  frxCustomDB, frxDsgnIntf, DB, frxUtils, frxEditQueryParams,
  SDEngine
{$IFDEF Delphi6}
, Variants, Dialogs
{$ENDIF};

{$I frx.inc}

implementation

uses
  frxSDComponents;

type
  TfrxDataBaseNameProperty = class(TfrxStringProperty)
  public
    procedure GetValues; override;
    function GetAttributes: TfrxPropertyAttributes; override;
  end; { TfrxDataBaseNameProperty }

  TfrxSessionNameProperty = class(TfrxStringProperty)
  public
    procedure GetValues; override;
    function GetAttributes: TfrxPropertyAttributes; override;
  end; { TfrxSessionNameProperty }

  TfrxTableNameProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
    procedure SetValue(const Value: String); override;
  end; { TfrxTableNameProperty }

  TfrxIndexNameProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
  end; { TfrxIndexNameProperty }

  TfrxIndexFieldNamesProperty = class(TfrxStringProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    procedure GetValues; override;
  end; { TfrxIndexFieldNamesProperty }

  TfrxMacrosProperty = class(TfrxClassProperty)
  public
    function GetAttributes: TfrxPropertyAttributes; override;
    function Edit: Boolean; override;
  end;

{ TfrxDataBaseNameProperty }

function TfrxDataBaseNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
 Result := [paMultiSelect, paValueList];
end; { GetAttributes }

procedure TfrxDataBaseNameProperty.GetValues;
var
  Session: TSDSession;

begin

 inherited;
 Session := Sessions.FindSession(TSDDataSet(TfrxCustomDataset(Component).DataSet).SessionName);
 if Session <> nil then
   Session.GetDatabaseNames(Values);

end; { GetValues }


{ TfrxSessionNameProperty }

function TfrxSessionNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
 Result := [paMultiSelect, paValueList];
end; { GetAttributes }

procedure TfrxSessionNameProperty.GetValues;
var
  nCou: Integer;

begin

 for nCou := 0 to Sessions.Count-1 do
   Values.Add(Sessions[nCou].SessionName);

end; { GetValues }


{ TfrxTableNameProperty }

function TfrxTableNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
 Result := [paMultiSelect, paValueList, paSortList];
end; { GetAttributes }

procedure TfrxTableNameProperty.GetValues;
begin

 inherited;

 try
   if TfrxSDTable(Component).DatabaseName <> '' then
     Session.GetTableNames(TfrxSDTable(Component).DatabaseName, '', False, Values);
 except;
 end; { try }

end; { GetValues }

procedure TfrxTableNameProperty.SetValue(const Value: String);
begin
 inherited;
 Designer.UpdateDataTree;
end; { SetValue }

{ TfrxIndexProperty }

function TfrxIndexNameProperty.GetAttributes: TfrxPropertyAttributes;
begin
 Result := [paMultiSelect, paValueList, paSortList];
end; { GetAttributes }

procedure TfrxIndexNameProperty.GetValues;
var
  i: Integer;

begin

 inherited;

 try
   with TfrxSDTable(Component).Table do
   if (TableName <> '') and (IndexDefs <> nil) then begin
     IndexDefs.Update;
     for i := 0 to IndexDefs.Count - 1 do
       if IndexDefs[i].Name <> '' then
         Values.Add(IndexDefs[i].Name);
   end; { if }
 except;
 end; { try }

end; { GetValues }


{ TfrxIndexFieldNamesProperty }

function TfrxIndexFieldNamesProperty.GetAttributes: TfrxPropertyAttributes;
begin
 Result := [paMultiSelect, paValueList, paSortList];
end; { GetAttributes }

procedure TfrxIndexFieldNamesProperty.GetValues;
var
  i: Integer;

begin

 inherited;

 try
   with TfrxSDTable(Component).Table do
     if (TableName <> '') and (IndexDefs <> nil) then begin

       IndexDefs.Update;
       for i := 0 to IndexDefs.Count - 1 do
         with IndexDefs[i] do
           if (Options * [ixExpression, ixDescending] = []) and (Fields <> '') then
             Values.Add(Fields);

     end; { if }
 except;
 end; { try }

end; { GetValues }


{ TfrxMacrosProperty }

function TfrxMacrosProperty.GetAttributes: TfrxPropertyAttributes;
begin
 Result := [paDialog, paReadOnly];
end; { GetAttributes }

function TfrxMacrosProperty.Edit: Boolean;
var
  q: TfrxSDMacroQuery;

begin

 Result := False;
 q := TfrxSDMacroQuery(Component);
 if q.Macros.Count <> 0 then
   with TfrxParamsEditorForm.Create(Designer) do begin
     Params := q.Macros;
     Result := (ShowModal = mrOk);
     if Result then
       q.UpdateParams;
     Free;
   end; { with }

end; { TfrxMacrosProperty }

initialization
  frxPropertyEditors.Register(TypeInfo(String), TfrxSDDataBase, 'SessionName',
    TfrxSessionNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDQuery, 'SessionName',
    TfrxSessionNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDQuery, 'DatabaseName',
    TfrxDataBaseNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDTable, 'SessionName',
    TfrxSessionNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDTable, 'DatabaseName',
    TfrxDataBaseNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDTable, 'TableName',
    TfrxTableNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDTable, 'IndexName',
    TfrxIndexNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDTable, 'IndexFieldNames',
    TfrxIndexFieldNamesProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDMacroQuery, 'SessionName',
    TfrxSessionNameProperty);

  frxPropertyEditors.Register(TypeInfo(String), TfrxSDMacroQuery, 'DatabaseName',
    TfrxDataBaseNameProperty);

  frxPropertyEditors.Register(TypeInfo(TfrxParams), TfrxSDMacroQuery, 'Macros',
    TfrxMacrosProperty);

end.
