{**********************************************}
{                                              }  
{              FastScript v1.9                 }
{     SQL Direct classes and functions         }
{                                              }
{   (c) 2003-2006 by Alexander Tzyganenko,     }
{              Fast Reports Inc                }
{                                              }
{ Copyright (c) 2005-2006 by Stalker SoftWare  }
{                                              }
{**********************************************}

unit fs_iSDRTTI;

interface

{$i fs.inc}

uses 
  SysUtils, Classes, fs_iinterpreter, fs_itools, fs_idbrtti, DB, SDEngine, 
  SDCommon;

type
  TfsSDRTTI = class(TComponent); // fake component

implementation

type
  TfsUpdateRecordEvent = class(TfsCustomEvent)
  public
    procedure DoEvent(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
    function GetMethod: Pointer; override;
  end;

type
  TFunctions = class(TfsRTTIModule)
  private
    function CallMethod(Instance: TObject; ClassType: TClass; const MethodName: String; Caller: TfsMethodHelper): Variant;
    function GetProp(Instance: TObject; ClassType: TClass;  const PropName: String): Variant;
  public
    constructor Create(AScript: TfsScript); override;
  end;

{ TfsUpdateRecordEvent }

procedure TfsUpdateRecordEvent.DoEvent(DataSet: TDataSet; UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
begin
 CallHandler([DataSet, Integer(UpdateKind), Integer(UpdateAction)]);
 UpdateAction := TUpdateAction(Handler.Params[2].Value);
end; { DoEvent }

function TfsUpdateRecordEvent.GetMethod: Pointer;
begin
 Result := @TfsUpdateRecordEvent.DoEvent;
end; { GetMethod }


{ TFunctions }

constructor TFunctions.Create(AScript: TfsScript);
begin

 inherited Create(AScript);

 with AScript do begin

   AddEnum('TSDServerType','stDB2, stInformix, stInterbase, stFirebird, stMySQL, stODBC, stOLEDB, stOracle, stSQLBase, stSQLServer, stSybase');
   AddEnum('TSDTransIsolation','tiDirtyRead, tiReadCommitted, tiRepeatableRead');
   AddEnum('TUpdateKind','ukModify, ukInsert, ukDelete');
   AddEnum('TUpdateAction','uaFail, uaAbort, uaSkip, uaRetry, uaApplied');

   with AddClass(TSDBlobStream, 'TStream') do begin
     AddConstructor('constructor Create(Field :TBlobField; Mode :TBlobStreamMode)', CallMethod);
     AddMethod('procedure LoadFromFile(const FileName :String)', CallMethod);
     AddMethod('procedure LoadFromStream(Stream :TStream)', CallMethod);
     AddMethod('procedure SaveToStream(Stream :TStream)', CallMethod);
     AddMethod('procedure SaveToFile(const FileName :String)', CallMethod);
   end;{ with }

   AddClass(TSDSession, 'TComponent');
   AddClass(TSDCustomDatabase, 'TComponent');

   with AddClass(TSDDatabase, 'TSDCustomDatabase') do begin
     AddMethod('procedure StartTransaction', CallMethod);
     AddMethod('procedure Commit', CallMethod);
     AddMethod('procedure Rollback', CallMethod);
     AddProperty('InTransaction', 'Boolean', GetProp, nil);
   end; { with }

   with AddClass(TSDDataSet, 'TDataSet') do begin
     AddMethod('function LocateNext(const KeyFields: string; const KeyValues: Variant;Options: TLocateOptions): Boolean', CallMethod);
     AddMethod('procedure OpenEmpty', CallMethod);
     AddMethod('procedure FetchAll', CallMethod);
     AddMethod('procedure RefreshRecordEx(const cRefreshSQL: string)', CallMethod);
     AddMethod('procedure SortRecords(const AFields, AscOrder, ACaseSensitive: string)', CallMethod);
     AddMethod('function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream', CallMethod);
     AddEvent('OnUpdateRecord', TfsUpdateRecordEvent);
   end; { with }

   with AddClass(TSDQuery, 'TSDDataSet') do begin
     AddProperty('ParamCount', 'Word', GetProp, nil);
     AddMethod('procedure ExecSQL', CallMethod);
     AddMethod('function ParamByName(const Value: string): TParam', CallMethod);
     AddMethod('procedure Prepare', CallMethod);
   end; { with }

   with AddClass(TSDMacroQuery, 'TSDQuery') do begin
     AddMethod('procedure ExpandMacros',CallMethod);
     AddMethod('function MacroByName(const Value: string): TParam', CallMethod);
   end; { with }

 end; { with }

end; { Create }

function TFunctions.CallMethod(Instance: TObject; ClassType: TClass; const MethodName: String; Caller: TfsMethodHelper): Variant;

 function IntToLocateOptions(i: Integer): TLocateOptions;
 begin
  Result := [];
  if (i and 1) <> 0 then
    Result := Result + [loCaseInsensitive];
  if (i and 2) <> 0 then
    Result := Result + [loPartialKey];
 end; { IntToLocateOptions }

begin

 Result := 0;

 if ClassType = TSDBlobStream then begin

   if MethodName = 'CREATE' then                   
     Result := Integer(TSDBlobStream.Create(TBlobField(Integer(Caller.Params[0])), Caller.Params[1]))
   else
   if MethodName = 'LOADFROMFILE' then
     TSDBlobStream(Instance).LoadFromFile(Caller.Params[0])
   else 
   if MethodName = 'LOADFROMSTREAM' then
     TSDBlobStream(Instance).LoadFromStream(TStream(Integer(Caller.Params[0])))
   else 
   if MethodName = 'SAVETOFILE' then
     TSDBlobStream(Instance).SaveToFile(Caller.Params[0])
   else 
   if MethodName = 'SAVETOSTREAM' then
     TSDBlobStream(Instance).SaveToStream(TStream(Integer(Caller.Params[0])))

 end else
 if ClassType = TSDDatabase then begin

   if MethodName = 'STARTTRANSACTION' then
     TSDDatabase(Instance).StartTransaction
   else
   if MethodName = 'COMMIT' then
     TSDDatabase(Instance).Commit
   else
   if MethodName = 'ROLLBACK' then
     TSDDatabase(Instance).Rollback

 end else
 if ClassType = TSDDataSet then begin

   if MethodName = 'LOCATENEXT' then
     Result := TSDDataSet(Instance).LocateNext(Caller.Params[0], Caller.Params[1], IntToLocateOptions(Caller.Params[2]))
   else
   if MethodName = 'OPENEMPTY' then
     TSDDataSet(Instance).OpenEmpty
   else
   if MethodName = 'FETCHALL' then
     TSDDataSet(Instance).FetchAll
   else
   if MethodName = 'REFRESHRECORDEX' then
     TSDDataSet(Instance).RefreshRecordEx(Caller.Params[0])
   else
   if MethodName = 'SORTRECORDS' then
     TSDDataSet(Instance).SortRecords(Caller.Params[0], Caller.Params[1], Caller.Params[2])
   else
   if MethodName = 'CREATEBLOBSTREAM' then
     Result := Integer(TSDDataSet(Instance).CreateBlobStream(TField(Integer(Caller.Params[0])), Caller.Params[1]));

 end else
 if ClassType = TSDQuery then begin

   if MethodName = 'EXECSQL' then
     TSDQuery(Instance).ExecSQL
   else 
   if MethodName = 'PARAMBYNAME' then
     Result := Integer(TSDQuery(Instance).ParamByName(Caller.Params[0]))
   else 
   if MethodName = 'PREPARE' then
     TSDQuery(Instance).Prepare

 end else 
 if ClassType = TSDMacroQuery then  begin

   if MethodName = 'EXPANDMACROS' then
     TSDMacroQuery(Instance).ExpandMacros
   else
   if MethodName = 'MACROBYNAME' then
     Result := Integer(TSDMacroQuery(Instance).MacroByName(Caller.Params[0]))

 end; { if }

end; { CallMethod }

function TFunctions.GetProp(Instance: TObject; ClassType: TClass; const PropName: String): Variant;
begin

 Result := 0;

 if ClassType = TSDDatabase then begin

   if PropName = 'INTRANSACTION' then
     Result := TSDDatabase(Instance).InTransaction

 end else
 if ClassType = TSDQuery then begin

   if PropName = 'PARAMCOUNT' then
     Result := TSDQuery(Instance).ParamCount

 end; { if }

end; { GetProp }

initialization
  fsRTTIModules.Add(TFunctions);

finalization
  fsRTTIModules.Remove(TFunctions);

end.
